@@ -12,6 +12,8 @@
 # Developed by Andres Cumsille, Andrea Rodriguez, Roberto E. Duran & Vicente Saona Urmeneta
 # For any code related query, contact: andrea.rodriguezdelherbe@rdm.ox.ac.uk, vicente.saona@sansano.usm.cl.
 
+#from create_tables import cogs_classif
+from .create_tables import cogs_classif
 
 from Bio import SeqIO
 import numpy as np
@@ -25,6 +27,7 @@ import pandas as pd
 import scripts.genbank2faa as genbank2faa
 global seek
 import pkg_resources
+import matplotlib.pyplot as plt
 
 
 __all__ = ['getArgs', 'listdir_r', 'ends_sorted', 'create_kar', 'create_feature', 'base_complete', 'base', 'get_categories', 'create_kar_complete', 'create_feature_complete',
@@ -91,7 +94,7 @@ def ends_sorted(ends):
 
 # Fuction for creating base KAR file that defines contig bands.
 # It considers that the genome is not complete.
-def create_kar(gbk_filename, output_folder, complete):
+def create_kar(gbk_filename, temp_folder, output_folder, complete):
 	
 	gbk_file = open(gbk_filename,"r")
 
@@ -109,13 +112,17 @@ def create_kar(gbk_filename, output_folder, complete):
 	inits = []
 	init = 0
 	end = ends[0]
-	for i in range(len(ends)-1):
-		inits.append(init)
-		new_ends.append(end)
-		init = end + 1
-		end = end + ends[i+1]
+	lengths = []
 	inits.append(init)
 	new_ends.append(end)
+	lengths.append(end-init)
+	for i in range(0,len(ends)-1):
+		
+		init = end + 1
+		end = end + ends[i+1]
+		inits.append(init)
+		new_ends.append(end)
+		lengths.append(end-init+1)
 		
 	lines = []
 	
@@ -123,20 +130,20 @@ def create_kar(gbk_filename, output_folder, complete):
 		for i in range(len(ends)):
 			line1 = "chr - chr"+ str(i+1) +" 1 " +str(inits[i])+" "+str(new_ends[i])+" black\n"
 			if i%2 == 0:
-				color = " white\n"
+				colour = " white\n"
 			else:
-				color = " black\n"
-			line2 = "band chr"+ str(i+1)+" band01 band01 "+str(inits[i])+" "+str(new_ends[i])+color
+				colour = " black\n"
+			line2 = "band chr"+ str(i+1)+" band01 band01 "+str(inits[i])+" "+str(new_ends[i])+colour
 			lines.append(line1)
 			lines.append(line2)
 
-		output_file = output_folder + "_bands.kar"
+		output_file = temp_folder + "_bands.kar"
 		with open(output_file, 'w') as output:
 			output.writelines(lines)
 			print(output_file+" created succesfully.")
 
 
-	return ends, inits, new_ends		
+	return ends, inits, new_ends, lengths		
 
 # Fuction for creating base KAR file that defines contig bands.
 # It considers that the genome is complete.
@@ -171,6 +178,15 @@ def new_loc(array, sizes_x):
 # Writting locations to file.
 def write_lines(locations, output_, chrx, locus, cogs, verbose = False):
 	lines = []
+	cogs_df = pd.DataFrame.from_dict(cogs)
+	categories = ['D','M','N','O','T','U','V','W','Y','Z','A','B','J','K','L','X','C','E','F','G','H','I','P','Q','R','S','None']
+	hist = pd.DataFrame(categories, columns=["cat"])
+	hist["freq"] = [0]*len(categories)
+	
+	chrms, counts = np.unique(chrx, return_counts=True)
+	for c in range(len(chrms)):
+		hist["chr"+str(chrms[c])] = [0]*len(categories)
+
 	#print(locations)		
 	for i in range(len(locations)):
 		#line = ["chr-Node_x_length_"+str(sizes_x[i])+"_cov_x"] + list(map(str, locations[i]))
@@ -179,13 +195,21 @@ def write_lines(locations, output_, chrx, locus, cogs, verbose = False):
 		elif len(cogs) == 0:
 			line = ["chr"+chrx[i]] + list(map(str, locations[i]))# + [locus[i]]
 		else:
-			line = ["chr"+chrx[i]] + list(map(str, locations[i]))# + [locus[i]] + [cogs[i]]
+			line = ["chr"+chrx[i]] + list(map(str, locations[i]))# + [locus[i]] + [cogs[i] if cogs[i] is not None else "None"]
+			cats = list(cogs[i]) if cogs[i] is not None else ["None"]
+			hist.loc[hist['cat'].isin(cats), 'freq'] += 1
+			hist.loc[hist['cat'].isin(cats), 'chr'+str(chrx[i])] += 1
 		lines.append(line)
 	with open(output_, 'w', newline='') as csvfile:
 		writer = csv.writer(csvfile, delimiter='\t', quotechar='|', quoting=csv.QUOTE_MINIMAL)
 		writer.writerows(lines)
 		if verbose:
 			print(output_,"created succesfully.")
+	
+	if len(cogs) == 0:
+		return None
+	else:
+		return hist
 
 def write_cog_files(locations, output, chrx, locus, cogs, verbose = False):
 	
@@ -197,13 +221,17 @@ def write_cog_files(locations, output, chrx, locus, cogs, verbose = False):
 	cogs_df[['loc_init','loc_end']] = pd.DataFrame(cogs_df.location.tolist(), index= cogs_df.index)
 	cogs_df["chr"] = chrx
 	cogs_df["locus"] = locus
-	cogs_df["main"] = cogs_df["category"].str[0]
+	cogs_df["main"] = cogs_df["category"].apply(lambda x: "None" if x is None else x[0]) #cogs_df["category"].str[0]
 	categories = map(str, cogs_df["main"].unique())
+	#hist = pd.DataFrame(columns = ["cat", "freq"])
 	
-	for c in categories:
+	for c in list(categories):
 		lines = []
 		subset = cogs_df.loc[cogs_df["main"] == c]
 		subset = subset.sort_values(["chr", "loc_init"], ascending=[True, True])
+		
+		#hist = hist.append({"cat": c, "freq": len(subset)}, ignore_index=True)
+		
 		for index, row in subset.iterrows():
 			line = ["chr"+row["chr"]] + list(map(str, row["location"]))
 			lines.append(line)
@@ -214,10 +242,52 @@ def write_cog_files(locations, output, chrx, locus, cogs, verbose = False):
 			if verbose:
 				print(filename, "created succesfully.")
 	
+	#return hist
+
+def postprocess(chroms):
+	
+	def get_tuples(init, chroms):
+		
+		n_chrom = int(len(chroms)/3)
+		
+		sum_p_c = np.array([], dtype='<U32')
+		sum_p_n = np.array([], dtype=np.float64)
+		sum_n_c = np.array([], dtype='<U32')
+		sum_n_n = np.array([], dtype=np.float64)
+		
+		for i in range(init,len(chroms),3):
+			
+			n_chrom = int(i/3)+1
+			
+			if len(chroms[i][0][0]) > 0 and len(chroms[i][1][0]) > 0:
+				sum_p_c = np.concatenate((sum_p_c, np.array([str(n_chrom)])))
+				sum_n_c = np.concatenate((sum_n_c, np.array([str(n_chrom)])))
+			elif len(chroms[i][0][0]) != len(chroms[i][1][0]):
+				sum_p_c = np.concatenate((sum_p_c, np.array([str(n_chrom)])))
+				sum_n_c = np.concatenate((sum_n_c, np.array([str(n_chrom)])))
+			else:	
+				sum_p_c = np.concatenate((sum_p_c, chroms[i][0][0]))
+				sum_n_c = np.concatenate((sum_n_c, chroms[i][1][0]))
+				
+			sum_p_n = np.array(np.concatenate((sum_p_n, chroms[i][0][1])))
+			sum_n_n = np.array(np.concatenate((sum_n_n, chroms[i][1][1])))
+		
+		return [(np.array(sum_p_c),np.array(sum_p_n)),(np.array(sum_n_c),np.array(sum_n_n))]
+	
+	new_chroms = []
+	n_chroms = int(len(chroms)/3)
+		
+	trnas = get_tuples(0, chroms)
+	rrnas = get_tuples(1, chroms)
+	cdss = get_tuples(2, chroms)
+	
+	return [trnas, rrnas, cdss]
+
+
 
 # Creates feature (CDS, tRNAm, or rRNA) files for CIRCOS.
 # It considers that the genome is not complete.
-def create_feature(gbk_filename, output, sizes, feat, cogs_dict=None, divided=False, verbose = False):
+def create_feature(gbk_filename, tmp, output, sizes, feat, cogs_dict=None, divided=False, verbose = False, complete=False):
 	
 	#chrx = '1'
 	
@@ -285,22 +355,36 @@ def create_feature(gbk_filename, output, sizes, feat, cogs_dict=None, divided=Fa
 	new_pos = new_loc(positives, sizes_p)
 	new_negs = 	new_loc(negatives, sizes_n)
 	
-	p_output = output + "_" + feat + "_pos.txt"
-	n_output = output + "_" + feat + "_neg.txt"
+	p_output = tmp + "_" + feat + "_pos.txt"
+	n_output = tmp + "_" + feat + "_neg.txt"
 	
 	if divided:
 		write_cog_files(new_pos, p_output, chrms_p, locus_p, cogs_p, verbose = verbose)
 		write_cog_files(new_negs, n_output, chrms_n, locus_n, cogs_n, verbose = verbose)
-	else:
-		write_lines(new_pos, p_output, chrms_p, locus_p, cogs_p, verbose = verbose)
-		write_lines(new_negs, n_output, chrms_n, locus_n, cogs_n, verbose = verbose)
+	#else:
+	hist1 = write_lines(new_pos, p_output, chrms_p, locus_p, cogs_p, verbose = verbose)
+	hist2 = write_lines(new_negs, n_output, chrms_n, locus_n, cogs_n, verbose = verbose)
 	
-	return(cogs_p, cogs_n)
+	if hist1 is not None and hist2 is not None:
+		hist1 = hist1.set_index("cat").add(hist2.set_index("cat"), fill_value=0).reset_index().replace("None", "Unclassified")
+		hist1.columns = ["COG Category", "Frequency"] + ["chr"+str(x) for x in sorted(list(set(chrms_p+chrms_n)))]
+	
+		#if hist1 is not None and hist2 is not None:
+		#	hist1 = hist1.set_index("cat").add(hist2.set_index("cat"), fill_value=0).reset_index().replace("None", "Unclassified")
+		#	hist1.columns = ["COG Category", "Frequency"] + ["chr"+str(x) for x in sorted(list(set(chrms_p+chrms_n)))]
+		#	ax = hist1.plot.bar(x="COG Category", y="Frequency", rot=90, legend=False)
+		#	for p in ax.patches:
+		#		ax.annotate(str(p.get_height()), (p.get_x()+p.get_width()/2, p.get_height()+5), ha='center', va='center')
+		#	plt.tight_layout()
+		#	ax.figure.savefig(output+"_COG_Histogram.png")
+		#	cogs_classif(hist1, output+"_COG_Classification.csv")
+	
+	return(cogs_p, cogs_n, chrms_p, chrms_n, hist1)
 	
 
 # Creates feature (CDS, tRNAm, or rRNA) files for CIRCOS.
 # It considers that the genome is complete.
-def create_feature_complete(gbk_filename, output, sizes, j, feat, cogs_dict=None, divided=False):
+def create_feature_complete(gbk_filename, output, sizes, j, feat, cogs_dict=None, divided=False, verbose = False):
 	
 	#chrx = '1'
 	
@@ -373,17 +457,32 @@ def create_feature_complete(gbk_filename, output, sizes, j, feat, cogs_dict=None
 		write_cog_files(new_pos, p_output, chrms_p, locus_p, cogs_p)
 		write_cog_files(new_negs, n_output, chrms_n, locus_n, cogs_n)		
 			 
+	#else:
+	#	write_lines(new_pos, p_output, chrms_p, locus_p, cogs_p)
+	#	write_lines(new_negs, n_output, chrms_n, locus_n, cogs_n)
+		
 	else:
-		write_lines(new_pos, p_output, chrms_p, locus_p, cogs_p)
-		write_lines(new_negs, n_output, chrms_n, locus_n, cogs_n)
+		hist1 = write_lines(new_pos, p_output, chrms_p, locus_p, cogs_p, verbose = verbose)
+		hist2 = write_lines(new_negs, n_output, chrms_n, locus_n, cogs_n, verbose = verbose)
+		if hist1 is not None and hist2 is not None:
+			hist1 = hist1.set_index("cat").add(hist2.set_index("cat"), fill_value=0).reset_index().replace("None", "Unclassified")
+			hist1.columns = ["COG Category", "Frequency"] + ["chr"+str(x) for x in sorted(list(set(chrms_p+chrms_n)))]
+			ax = hist1.plot.bar(x="COG Category", y="Frequency", rot=90, legend=False)
+			#ax.set_ylabel("CDS Frequency")
+			for p in ax.patches:
+				ax.annotate(str(p.get_height()), (p.get_x()+p.get_width()/2, p.get_height()+5), ha='center', va='center')
+			plt.tight_layout()
+			ax.figure.savefig(output+"_COG_Histogram.png")
+			cogs_classif(hist1, output+"_COG_Classification.csv")
 	
-	return
+	return(cogs_p, cogs_n, chrms_p, chrms_n)
+	
+	#return
 
 
 # Function to predict COG categories with DeepNOG
 def get_categories(gbk_file, output, deepnog_confidence = 0):
 	
-	
 	# Check if deepnog is installed
 	
 	if which("deepnog") == None:
@@ -464,7 +563,7 @@ def get_categories(gbk_file, output, deepnog_confidence = 0):
 
 
 # Base pipeline for complete genome.
-def base_complete(gbk_file, output, cds, trna, get_cats, divided, k, init, end, sizes):
+def base_complete(gbk_file, tmp, output, cds, trna, get_cats, divided, k, init, end, sizes):
 	
 	flag = True
 	
@@ -494,7 +593,7 @@ def base_complete(gbk_file, output, cds, trna, get_cats, divided, k, init, end,
 			create_feature_complete(gbk_file, output, sizes, k, "CDS", cogs_dict, divided)
 
 # Base pipeline for non-complete genome.	
-def base(gbk_file, output, cds, trna, get_cats, divided, complete, rrna = False, deepnog_confidence = 0, verbose = False):
+def base(gbk_file, tmp, output, cds, trna, get_cats, divided, complete, rrna = False, deepnog_confidence = 0, verbose = False):
 	
 	flag = True
 
@@ -505,7 +604,7 @@ def base(gbk_file, output, cds, trna, get_cats, divided, complete, rrna = False,
 		print("Error: Categories can only be predicted for CDS. Please enter output file paths for both CDS.") 
 		flag = False
 	elif get_cats:
-		cogs_dict = get_categories(gbk_file, output, deepnog_confidence)
+		cogs_dict = get_categories(gbk_file, tmp, deepnog_confidence)
 	else:
 		cogs_dict = None
 		
@@ -515,17 +614,33 @@ def base(gbk_file, output, cds, trna, get_cats, divided, complete, rrna = False,
 	
 	if flag:
 		
-		sizes, _, _ = create_kar(gbk_file, output, complete)
-			
+		sizes, _, _, lengths = create_kar(gbk_file, tmp, output, complete)
+		chrms = []
+		hist = None
+		
 		if trna:
-			create_feature(gbk_file, output, sizes, "tRNA", verbose = verbose)
+			_ , _, chrms_tp, chrms_tn, _ = create_feature(gbk_file, tmp, output, sizes, "tRNA", verbose = verbose, complete=complete)
+			#print("\nHIST TRNA\n")
+			#print(hist)
+			chrms_t = [np.unique(chrms_tp, return_counts=True), np.unique(chrms_tn, return_counts=True)]
+			chrms.append(chrms_t)
 		if rrna:
-			create_feature(gbk_file, output, sizes, "rRNA", verbose = verbose)
+			_ , _, chrms_rp, chrms_rn, _ = create_feature(gbk_file, tmp, output, sizes, "rRNA", verbose = verbose, complete=complete)
+			#print("\nHIST RRNA\n")
+			#print(hist)
+			chrms_r = [np.unique(chrms_rp, return_counts=True), np.unique(chrms_rn, return_counts=True)]
+			chrms.append(chrms_r)
 		if cds:
-			create_feature(gbk_file, output, sizes, "CDS", cogs_dict, verbose = verbose)
+			_ , _, chrms_cp, chrms_cn, hist = create_feature(gbk_file, tmp, output, sizes, "CDS", cogs_dict, verbose = verbose, complete=complete)
+			
+			chrms_c = [np.unique(chrms_cp, return_counts=True), np.unique(chrms_cn, return_counts=True)]
+			chrms.append(chrms_c)
+			
 		if divided:
-			cogs_p, cogs_n = create_feature(gbk_file, output, sizes, "CDS", cogs_dict, divided, verbose = verbose)
-	return ((sizes, cogs_p, cogs_n))
+			cogs_p, cogs_n, _, _, hist = create_feature(gbk_file, tmp, output, sizes, "CDS", cogs_dict, divided, verbose = verbose, complete=complete)
+
+			
+	return ((sizes, cogs_p, cogs_n, lengths, chrms, hist))
 
 def createRaw():
 	gbk_file, output, cds, trna, rrna, get_cats, divided, complete = getArgs()[:]
@@ -590,8 +705,11 @@ if __name__ == '__main__':
 	
 	if output == "":
 		output = gbk_name + "/"
+		tmp = gbk_name + "-temp/"
 	elif output[-1] != "/":
+		tmp = output + "-temp/"
 		output = output + "/"
+		
 	
 	if not os.path.isdir(output):
 		os.mkdir(output)
@@ -606,7 +724,7 @@ if __name__ == '__main__':
 		except:
 			gbk_name = gbk_file.split(".")[-2]	
 			
-		sizes, inits, ends = create_kar(gbk_file, output, complete)
+		sizes, inits, ends, lengths = create_kar(gbk_file, tmp, output, complete)
 
 		for k, rec in enumerate(SeqIO.parse(gbk, "genbank")):
 			
@@ -623,7 +741,7 @@ if __name__ == '__main__':
 			
 			output_ = output_ + gbk_name + '_' + str(k+1)
 			
-			base_complete(filename, output_, cds, trna, get_cats, divided, k, inits[k], ends[k], sizes)
+			base_complete(filename, tmp, output_, cds, trna, get_cats, divided, k, inits[k], ends[k], sizes)
 	
 	else:
 		
